' SED_PBWIN - Use the PBWIN compiler
#COMPILE EXE
#DIM ALL
#INCLUDE ONCE "GLFW.INC"
#INCLUDE ONCE "GDIPUTILS.INC"

%MAX_PARTICLES = 1000   ' Number of particles to create

TYPE particles
  active AS LONG    ' Active (Yes/No)
  life AS SINGLE    ' Particle life
  fade AS SINGLE    ' Fade speed
  r AS SINGLE       ' Red value
  g AS SINGLE       ' Green value
  b AS SINGLE       ' Blue value
  x AS SINGLE       ' X Position
  y AS SINGLE       ' Y Position
  z AS SINGLE       ' Z Position
  xi AS SINGLE      ' X Direction
  yi AS SINGLE      ' Y Direction
  zi AS SINGLE      ' Z Direction
  xg AS SINGLE      ' X Gravity
  yg AS SINGLE      ' Y Gravity
  zg AS SINGLE      ' Z Gravity
END TYPE

' =======================================================================================
' Main
' =======================================================================================
FUNCTION PBMAIN () AS LONG

   LOCAL hr, nWidth, nHeight, running, frames, x, y AS LONG
   LOCAL t, t0, fps AS DOUBLE
   LOCAL szTitlestr AS ASCIIZ * 200
   LOCAL strTextureData AS STRING
   LOCAL TextureWidth, TextureHeight AS LONG
   DIM   TextureHandle AS DWORD
   LOCAL sx, sy, sz AS SINGLE

   LOCAL rainbow AS LONG      ' Rainbow mode?
   LOCAL sp AS LONG           ' Spacebar pressed?
   LOCAL rp AS LONG           ' Enter key pressed?
   LOCAL slowdown AS SINGLE   ' Slow down particles
   LOCAL xspeed AS SINGLE     ' Base x speed (to allow keyboard direction of tail)
   LOCAL yspeed AS SINGLE     ' Base y speed (to allow keyboard direction of tail)
   LOCAL zoom AS SINGLE       ' Used to zoom out
   LOCAL i AS LONG            ' Loop variable
   LOCAL clr AS DWORD         ' Current color selection
   LOCAL delay AS DWORD       ' Rainbow effec delay

   slowdown = 2.0!
   zoom = -40.0!

   DIM   particle (%MAX_PARTICLES - 1) AS particles   ' Particle array (room for particle info)
   DIM   colors (11, 2) AS SINGLE

   colors ( 0, 0) = 1.0!  : colors ( 0, 1) = 0.5!  : colors ( 0, 2) = 0.5!
   colors ( 1, 0) = 1.0!  : colors ( 1, 1) = 0.75! : colors ( 1, 2) = 0.5!
   colors ( 2, 0) = 1.0!  : colors ( 2, 1) = 1.0!  : colors ( 2, 2) = 0.5!
   colors ( 3, 0) = 0.75! : colors ( 3, 1) = 1.0!  : colors ( 3, 2) = 0.5!
   colors ( 4, 0) = 0.5!  : colors ( 4, 1) = 1.0!  : colors ( 4, 2) = 0.5!
   colors ( 5, 0) = 0.5!  : colors ( 5, 1) = 1.0!  : colors ( 5, 2) = 0.75!
   colors ( 6, 0) = 0.5!  : colors ( 6, 1) = 1.0!  : colors ( 6, 2) = 1.0!
   colors ( 7, 0) = 0.5!  : colors ( 7, 1) = 0.75! : colors ( 7, 2) = 1.0!
   colors ( 8, 0) = 0.5!  : colors ( 8, 1) = 0.5!  : colors ( 8, 2) = 1.0!
   colors ( 9, 0) = 0.75! : colors ( 9, 1) = 0.5!  : colors ( 9, 2) = 1.0!
   colors (10, 0) = 1.0!  : colors (10, 1) = 0.5!  : colors (10, 2) = 1.0!
   colors (11, 0) = 1.0!  : colors (11, 1) = 0.5! :  colors (11, 2) = 0.75!

   ' Initialize GLFW
   glfwInit

   ' Open OpenGL window
   IF ISFALSE glfwOpenWindow(640, 480, 0, 0, 0, 0, 16, 0, %GLFW_WINDOW) THEN
      glfwTerminate
      EXIT FUNCTION
   END IF

   ' Load bitmap texture from disk
   hr = GdiPlusLoadTexture("particle.bmp", TextureWidth, TextureHeight, strTextureData, %TRUE)

   ' Assign handle
   glGenTextures 1, TextureHandle

   ' Create linear filtered texture
   glBindTexture %GL_TEXTURE_2D, TextureHandle
   glTexParameteri %GL_TEXTURE_2D, %GL_TEXTURE_MAG_FILTER, %GL_LINEAR
   glTexParameteri %GL_TEXTURE_2D, %GL_TEXTURE_MIN_FILTER, %GL_LINEAR
   glTexImage2D %GL_TEXTURE_2D, 0, 3, TextureWidth, TextureHeight, 0, _
                %GL_RGBA, %GL_UNSIGNED_BYTE, BYVAL STRPTR(strTextureData)

   ' Select smooth shading
   glShadeModel %GL_SMOOTH
   ' Specify clear values for the color buffers
   glClearColor 0.0!, 0.0!, 0.0!, 0.0!
   ' Disable depth testing
   glDisable %GL_DEPTH_TEST
   ' Enable blending
   glEnable %GL_BLEND
   ' Type of blending to perform
   glBlendFunc %GL_SRC_ALPHA, %GL_ONE
	' Really nice perspective calculations
   glHint %GL_PERSPECTIVE_CORRECTION_HINT, %GL_NICEST
   ' Really nice point smoothing
   glHint %GL_POINT_SMOOTH_HINT, %GL_NICEST
   ' Enable texture maping
   glEnable %GL_TEXTURE_2D
   ' Select texture
   glBindTexture %GL_TEXTURE_2D, TextureHandle

   ' Initializes all the particles
   FOR i = 0 TO %MAX_PARTICLES -1
      particle(i).active = %True                             ' Make all the particles active
      particle(i).life = 1.0!                                ' Give all the particles full life
      particle(i).fade = (RND * 100) / 1000.0! + 0.003!  ' Random fade speed
      particle(i).r = colors(i * (12 \ %MAX_PARTICLES), 0)   ' Select red rainbow color
      particle(i).g = colors(i * (12 \ %MAX_PARTICLES), 1)   ' Select red rainbow color
      particle(i).b = colors(i * (12 \ %MAX_PARTICLES), 2)   ' Select red rainbow color
      particle(i).xi = ((RND * 50) - 26.0!) * 10.0!      ' Random speed on x axis
      particle(i).yi = ((RND * 50) - 25.0!) * 10.0!      ' Random speed on y axis
      particle(i).zi = ((RND * 50) - 25.0!) * 10.0!      ' Random speed on z axis
      particle(i).xg = 0.0!                                  ' Set horizontal pull to zero
      particle(i).yg = -0.8!                                 ' Set vertical pull downward
      particle(i).zg = 0.0!                                  ' Set pull on z axis to zero
   NEXT

   ' Enable sticky keys
   glfwEnable %GLFW_STICKY_KEYS

   ' Disable vertical sync (on cards that support it)
   glfwSwapInterval 0

   ' Main loop
   running = %TRUE
   frames = 0
   t0 = glfwGetTime

   DO WHILE running

      ' Get time and mouse position
      t = glfwGetTime
      glfwGetMousePos x, y
      ' Calculate and display FPS (frames per second)
      IF t - t0 > 1.0! OR frames = 0 THEN
         fps = frames / (t-t0)
         wsprintf szTitlestr, "NEHE Lesson 19 (%i FPS)", BYVAL fps
         glfwSetWindowTitle szTitlestr
         t0 = t
         frames = 0
      END IF
      frames = frames + 1

      '  Get window size (may be different than the requested size)
      glfwGetWindowSize nWidth, nHeight
      IF nHeight <= 0 THEN nHeight = 1

      ' Set viewport
      glViewport 0, 0, nWidth, nHeight

      ' Clear the screen and the depth buffers
      glClear %GL_COLOR_BUFFER_BIT OR %GL_DEPTH_BUFFER_BIT

      ' Select and setup the projection matrix
      glMatrixMode %GL_PROJECTION
      glLoadIdentity

      ' Calculate the aspect ratio of the window
      gluPerspective 45.0!, nWidth/nHeight, 0.1!, 100.0!

      ' Select the modelview matrix
      glMatrixMode %GL_MODELVIEW
      glLoadIdentity

      ' Loop through all the particles
      FOR i = 0 TO %MAX_PARTICLES - 1
         IF particle(i).active THEN           ' If the particle is active
            sx = particle(i).x                ' Grab our particle x position
            sy = particle(i).y                ' Grab our particle y position
            sz = particle(i).z + zoom         ' Particle z pos + zoom

            ' draw the particle using our rgb values, fade the particle based on it's life
            glColor4f particle(i).r, particle(i).g, particle(i).b, particle(i).life
            glBegin %GL_TRIANGLE_STRIP                                ' build quad from a triangle strip
               glTexCoord2d 1, 1 : glVertex3f sx + 0.5!, sy + 0.5!, sz  ' Top right
               glTexCoord2d 0, 1 : glVertex3f sx - 0.5!, sy + 0.5!, sz  ' Top left
               glTexCoord2d 1, 0 : glVertex3f sx + 0.5!, sy - 0.5!, sz  ' Bottom right
               glTexCoord2d 0, 0 : glVertex3f sx - 0.5!, sy - 0.5!, sz  ' Bottom left
            glEnd                                                     ' Done building triangle strip

            particle(i).x = particle(i).x + particle(i).xi / (slowdown * 1000)  ' Move on the x axis by x speed
            particle(i).y = particle(i).y + particle(i).yi / (slowdown * 1000)  ' Move on the y axis by y speed
            particle(i).z = particle(i).z + particle(i).zi / (slowdown * 1000)  ' Move on the z axis by z speed

            particle(i).xi = particle(i).xi + particle(i).xg          ' Take pull on x axis into account
            particle(i).yi = particle(i).yi + particle(i).yg          ' Take pull on y axis into account
            particle(i).zi = particle(i).zi + particle(i).zg          ' Take pull on z axis into account
            particle(i).life = particle(i).life - particle(i).fade    ' Reduce particles life by 'fade'

            IF particle(i).life < 0.0! THEN                           ' If particle is burned out
               particle(i).life = 1.0!                                ' Give it new life
               particle(i).fade = (RND * 100) / 1000.0! + 0.003!  ' Random fade value
               particle(i).x = 0.0!                                   ' Center on x axis
               particle(i).y = 0.0!                                   ' Center on y axis
               particle(i).z = 0.0!                                   ' Center on z axis
               particle(i).xi = xspeed + ((RND * 60) - 32.0!)     ' X axis speed and direction
               particle(i).yi = yspeed + ((RND * 60) - 30.0!)     ' Y axis speed and direction
               particle(i).zi = ((RND * 60) - 30.0!)              ' Z axis speed and direction
               particle(i).r = colors(clr, 0)                         ' Select red from color table
               particle(i).g = colors(clr, 1)                         ' Select green from color table
               particle(i).b = colors(clr, 2)                         ' Select blue from color table
            END IF
         END IF
      NEXT

      ' Swap buffers
      glfwSwapBuffers

      ' Speed up particles
      IF glfwGetKey(%GLFW_KEY_KP_ADD) = %GLFW_PRESS AND slowdown > 1.0! THEN slowdown = slowdown - 0.01!

      ' Slow down particles
      IF glfwGetKey(%GLFW_KEY_KP_SUBTRACT) = %GLFW_PRESS AND slowdown < 4.0! THEN slowdown = slowdown + 0.01!

      ' Zoom in
      IF glfwGetKey(%GLFW_KEY_PAGEUP) THEN zoom = zoom + 0.1!

      ' Zoom out
      IF glfwGetKey(%GLFW_KEY_PAGEDOWN) THEN zoom = zoom - 0.1!

      ' Enter key
      IF glfwGetKey(%GLFW_KEY_ENTER) = %GLFW_PRESS AND ISFALSE rp THEN
         rp = %TRUE
         rainbow = NOT rainbow
      END IF
      IF glfwGetKey(%GLFW_KEY_ENTER) <> %GLFW_PRESS THEN rp = %FALSE

      ' Space or rainbow mode
      IF (glfwGetKey(32) = %GLFW_PRESS AND ISFALSE sp) OR (rainbow AND (delay > 25)) THEN
         IF glfwGetKey(32) = %GLFW_PRESS THEN rainbow = %FALSE   ' Disable rainbow mode
         sp = %TRUE                 ' Set flag telling us space is pressed
         delay = 0                  ' Reset the rainbow color cycling delay
         clr = clr + 1              ' Change the particle color
         IF clr > 11 THEN clr = 0   ' If color is too high, reset it
      END IF
      IF glfwGetKey(32) <> %GLFW_PRESS THEN sp = %FALSE

      ' If up arrow and y speed is less than 200 increase upward speed
      IF glfwGetKey(%GLFW_KEY_UP) = %GLFW_PRESS AND yspeed < 200 THEN yspeed = yspeed + 1.0!
      ' If down arrow and y speed is greater than -200 increase downward speed
      IF glfwGetKey(%GLFW_KEY_DOWN) = %GLFW_PRESS AND yspeed > -200 THEN yspeed = yspeed - 1.0!
      ' If right arrow and x speed is less than 200 increase speed to the right
      IF glfwGetKey(%GLFW_KEY_RIGHT) = %GLFW_PRESS AND xspeed < 200 THEN xspeed = xspeed + 1.0!
      ' If left arrow and x speed is greater than -200 increase speed to the left
      IF glfwGetKey(%GLFW_KEY_LEFT) = %GLFW_PRESS AND xspeed > - 200 THEN xspeed = xspeed - 1.0!

      ' Check if the ESC key was pressed or the window was closed
      running = NOT glfwGetKey(%GLFW_KEY_ESC) AND glfwGetWindowParam(%GLFW_OPENED)

   LOOP

   ' Delete the texture
   glDeleteTextures 1, TextureHandle

   ' Close OpenGL window and terminate GLFW
   glfwTerminate

END FUNCTION
' =======================================================================================
